@@ -11,11 +11,29 @@ from tensorboard_termplot.backend.matplotlib_plot import MatplotlibPlot
 from tensorboard_termplot.backend.terminal_plot import TerminalPlot
 
 
-def pair_of_int(arg):
+def pair_of_num(arg):
     arg = arg.split(",")
     if len(arg) != 2:
-        raise ValueError("It must be in the format of a,b (e.g. 300,20)")
-    return tuple(map(int, arg))
+        raise ValueError("It must be in the format of a,b (e.g. 30.0,2.2)")
+    return tuple(map(float, arg))
+
+
+def pair_of_int(arg):
+    return tuple(map(int, pair_of_num(arg)))
+
+
+def pair_of_num_assign_pair_of_num(arg):
+    arg = arg.split("=")
+    if len(arg) != 2:
+        raise ValueError("It must be in the format of a,b=c,d (e.g. 30.0,20=30.2,45)")
+    return tuple(map(pair_of_num, arg))
+
+
+def between_zero_and_one(arg):
+    arg = float(arg)
+    if not (0 <= arg <= 1):
+        raise ValueError()
+    return arg
 
 
 parser = argparse.ArgumentParser()
@@ -23,6 +41,27 @@ parser.add_argument(
     "folder", metavar="FOLDER", type=str, help="Folder of a tensorboard runs"
 )
 
+# plotting generic flags
+parser.add_argument(
+    "--backend",
+    default="plotext",
+    help="Set the plotting backend",
+    choices=["plotext", "matplotlib"],
+    type=str,
+)
+parser.add_argument(
+    "-m", "--matplotlib", help="Alias of --backend matplotlib", action="store_true",
+)
+parser.add_argument(
+    "--plotsize",
+    help="Manually set the size of each subplot, e.g., 50,20.",
+    metavar="WIDTH,HEIGHT",
+    type=pair_of_int,
+)
+parser.add_argument(
+    "-c", "--consolidate", action="store_true", help="Consolidate based on prefix."
+)
+
 # canvas flags
 parser.add_argument(
     "--canvas-color",
@@ -40,6 +79,7 @@ parser.add_argument(
     type=str,
     help="sets the (full-ground) color of the axes ticks and of the grid lines.",
 )
+parser.add_argument("--grid", action="store_true", help="Show grid.")
 parser.add_argument("--colorless", action="store_true", help="Remove color.")
 parser.add_argument(
     "-d",
@@ -49,15 +89,52 @@ parser.add_argument(
     "axes-color to black, and setting ticks-color to red. Can be overwritten "
     "individually.",
 )
-# flags
-parser.add_argument("--grid", action="store_true", help="Show grid.")
 parser.add_argument(
-    "--backend",
-    default="plotext",
-    help="Set the plotting backend",
-    choices=["plotext", "matplotlib"],
-    type=str,
+    "--no-iter-color",
+    action="store_true",
+    help="Stop iterating through different colors per plot.",
 )
+parser.add_argument(
+    "--force-label",
+    action="store_true",
+    help="Force showing label even for plot with one series.",
+)
+
+# auto-refresh related flag
+parser.add_argument(
+    "-f",
+    "--follow",
+    action="store_true",
+    help="Run in a loop to update display periodic.",
+)
+parser.add_argument(
+    "-n",
+    "--interval",
+    type=float,
+    metavar="secs",
+    default=5,
+    help="seconds to wait between updates",
+)
+
+# filtering for stats name
+parser.add_argument(
+    "-w",
+    "--whitelist",
+    type=str,
+    nargs="+",
+    metavar="keyword",
+    help="Keyword that the stat must contain for it to be plotted, case sensitive.",
+)
+parser.add_argument(
+    "-b",
+    "--blacklist",
+    type=str,
+    nargs="+",
+    metavar="keyword",
+    help="Keyword that the stat must not contain for it to be plotted, case sensitive.",
+)
+
+# axis flag
 parser.add_argument(
     "-x",
     "--xaxis-type",
@@ -66,12 +143,6 @@ parser.add_argument(
     choices=["step", "walltime"],
     type=str,
 )
-parser.add_argument(
-    "--plotsize",
-    help="Manually set the size of each subplot, e.g., 50,20.",
-    metavar="WIDTH,HEIGHT",
-    type=pair_of_int,
-)
 parser.add_argument(
     "--xlog",
     help="Set the list of subplots to use log scale in x-axis",
@@ -100,59 +171,48 @@ parser.add_argument(
     type=pair_of_int,
     nargs="*",
 )
+parser.add_argument(
+    "--xlim",
+    help="Set the list of xlim for the specified subplot.",
+    metavar="row,col=min,max",
+    type=pair_of_num_assign_pair_of_num,
+    nargs="+",
+)
+parser.add_argument(
+    "--ylim",
+    help="Set the list of ylim for the specified subplot.",
+    metavar="row,col=min,max",
+    type=pair_of_num_assign_pair_of_num,
+    nargs="+",
+)
+
+# matplotlib backend specific
 parser.add_argument(
     "--as-raw-bytes", action="store_true", help="Writes the raw image bytes to stdout.",
 )
-parser.add_argument(
-    "--force-label",
-    action="store_true",
-    help="Force showing label even for plot with one series.",
-)
-parser.add_argument(
-    "--no-iter-color",
-    action="store_true",
-    help="Stop iterating through different colors per plot.",
-)
-parser.add_argument(
-    "-c", "--consolidate", action="store_true", help="Consolidate based on prefix."
-)
-parser.add_argument(
-    "-f",
-    "--follow",
-    action="store_true",
-    help="Run in a loop to update display periodic.",
-)
 parser.add_argument(
     "--timg",
     action="store_true",
     help="Pass output to timg for matplotlib backend, implies --as-raw-bytes.",
 )
 parser.add_argument(
-    "-n",
-    "--interval",
-    type=float,
-    metavar="secs",
-    default=5,
-    help="seconds to wait between updates",
-)
-# filtering for stats name
-parser.add_argument(
-    "-w",
-    "--whitelist",
-    type=str,
-    nargs="+",
-    metavar="keyword",
-    help="Keyword that the stat must contain for it to be plotted, case sensitive.",
+    "-s",
+    "--smooth",
+    metavar="0-1",
+    const=0.05,
+    nargs="?",
+    type=between_zero_and_one,
+    help="A value from 0 to 1 as a smoothing factor.",
 )
 parser.add_argument(
-    "-b",
-    "--blacklist",
-    type=str,
-    nargs="+",
-    metavar="keyword",
-    help="Keyword that the stat must not contain for it to be plotted, case sensitive.",
+    "--smooth-poly-order",
+    metavar="poly-order",
+    default=3,
+    type=int,
+    help="Polynomial order for the savgol smoothing algorithm.",
 )
 
+# plotext backend specific
 parser.add_argument(
     "--terminal-width", type=int, help="Manually set the terminal width.",
 )
@@ -165,6 +225,36 @@ class EmptyEventFileError(Exception):
     pass
 
 
+def ensure_odd(x: int, roundup: bool):
+    if x % 2 == 0:
+        if roundup:
+            return x + 1
+        else:
+            return x - 1
+    return x
+
+
+def apply_smoothing(y_vals, smoothing_factor: float, poly_order: int):
+    from scipy.signal import savgol_filter
+
+    # factor controls the window size, with factor 0 being the min and 1
+    # being the max
+    # window size must be odd, and greater than the polynomial order
+    min_win_size = poly_order + 1
+    max_win_size = len(y_vals)
+
+    min_win_size = ensure_odd(min_win_size, roundup=True)
+    max_win_size = ensure_odd(max_win_size, roundup=False)
+    assert min_win_size < max_win_size
+
+    # apply the user-supplied factor
+    win_size = int(min_win_size + smoothing_factor * (max_win_size - min_win_size))
+
+    win_size = ensure_odd(win_size, roundup=False)
+
+    return savgol_filter(y_vals, win_size, poly_order)
+
+
 def get_consolidated_stats_list(
     args: argparse.ArgumentParser, scalar_names: List[str]
 ) -> Dict[str, List[str]]:
@@ -222,6 +312,11 @@ def _plot_for_one_run(plotter: Plotter, run_dict: Dict, col_num: int):
         for scalar_name, color in zip(scalar_names, colors):
             series = np.array(run_dict["ea"].Scalars(scalar_name))
             wall_t, steps, vals = series.T
+            if plotter.args.smooth:
+                vals = apply_smoothing(
+                    vals, plotter.args.smooth, plotter.args.smooth_poly_order
+                )
+
             if plotter.args.xaxis_type == "step":
                 x = steps
             elif plotter.args.xaxis_type == "walltime":
@@ -341,7 +436,9 @@ def main(args):
 def run():
     try:
         _args = parser.parse_args()
+        # handles alias of options
         if _args.dark_theme:
+            # only set values when unset, such that they can be overridden
             if _args.canvas_color is None:
                 _args.canvas_color = "black"
             if _args.axes_color is None:
@@ -350,6 +447,8 @@ def run():
                 _args.ticks_color = "white"
         if _args.timg:
             _args.as_raw_bytes = True
+        if _args.matplotlib:
+            _args.backend = "matplotlib"
         main(_args)
     except KeyboardInterrupt:
         pass
